@@ -7,7 +7,7 @@ Released under the GNU GPL version 3 or later
 
 '''
 
-import sys, os, time, socket
+import sys, os, time, socket, signal
 import fnmatch, errno, threading
 import serial, Queue, select
 import traceback
@@ -127,30 +127,34 @@ class MPState(object):
               MPSetting('mavfwd', bool, True, 'Allow forwarded control'),
               MPSetting('mavfwd_rate', bool, False, 'Allow forwarded rate control'),
               MPSetting('shownoise', bool, True, 'Show non-MAVLink data'),
+              MPSetting('baudrate', int, opts.baudrate, 'baudrate for new links', range=(0,10000000), increment=1),
+              MPSetting('rtscts', bool, opts.rtscts, 'enable flow control'),
+              MPSetting('select_timeout', float, 0.01, 'select timeout'),
 
               MPSetting('altreadout', int, 10, 'Altitude Readout',
                         range=(0,100), increment=1, tab='Announcements'),
               MPSetting('distreadout', int, 200, 'Distance Readout', range=(0,10000), increment=1),
 
               MPSetting('moddebug', int, 0, 'Module Debug Level', range=(0,3), increment=1, tab='Debug'),
+              MPSetting('compdebug', int, 0, 'Computation Debug Mask', range=(0,3), tab='Debug'),
               MPSetting('flushlogs', bool, False, 'Flush logs on every packet'),
               MPSetting('requireexit', bool, False, 'Require exit command'),
+              MPSetting('wpupdates', bool, True, 'Announce waypoint updates'),
 
               MPSetting('basealt', int, 0, 'Base Altitude', range=(0,30000), increment=1, tab='Altitude'),
               MPSetting('wpalt', int, 100, 'Default WP Altitude', range=(0,10000), increment=1),
               MPSetting('rallyalt', int, 90, 'Default Rally Altitude', range=(0,10000), increment=1),
               MPSetting('terrainalt', str, 'Auto', 'Use terrain altitudes', choice=['Auto','True','False']),
-              MPSetting('wpupdates', bool, True, 'Show waypoint updates'),
               MPSetting('rally_breakalt', int, 40, 'Default Rally Break Altitude', range=(0,10000), increment=1),
-              MPSetting('rally_flags', int, 0, 'Default Rally Flags`', range=(0,10000), increment=1),
-              MPSetting('baudrate', int, opts.baudrate, 'baudrate for new links', range=(0,10000000), increment=1),
-              MPSetting('rtscts', bool, opts.rtscts, 'enable flow control'),
-              MPSetting('select_timeout', float, 0.01, 'select timeout')]
-            )
+              MPSetting('rally_flags', int, 0, 'Default Rally Flags', range=(0,10000), increment=1)
+            ])
 
         self.completions = {
-            "script" : ["(FILENAME)"],
-            "set"    : ["(SETTING)"]
+            "script"         : ["(FILENAME)"],
+            "set"            : ["(SETTING)"],
+            "module"    : ["list",
+                           "load (AVAILMODULES)",
+                           "<unload|reload> (LOADEDMODULES)"]
             }
 
         self.status = MPStatus()
@@ -180,6 +184,8 @@ class MPState(object):
     
     def master(self):
         '''return the currently chosen mavlink master object'''
+        if len(self.mav_master) == 0:
+              return None
         if self.settings.link > len(self.mav_master):
             self.settings.link = 1
 
@@ -469,7 +475,10 @@ def process_master(m):
     if len(s) == 0:
         time.sleep(0.1)
         return
-    
+
+    if (mpstate.settings.compdebug & 1) != 0:
+        return
+
     if mpstate.logqueue_raw:
         mpstate.logqueue_raw.put(str(s))
 
@@ -578,7 +587,7 @@ def open_logs():
     # use a separate thread for writing to the logfile to prevent
     # delays during disk writes (important as delays can be long if camera
     # app is running)
-    t = threading.Thread(target=log_writer)
+    t = threading.Thread(target=log_writer, name='log_writer')
     t.daemon = True
     t.start()
 
@@ -625,6 +634,9 @@ def periodic_tasks():
     if mpstate.status.setup_mode:
         return
 
+    if (mpstate.settings.compdebug & 2) != 0:
+        return
+
     if mpstate.settings.heartbeat != 0:
         heartbeat_period.frequency = mpstate.settings.heartbeat
 
@@ -699,13 +711,19 @@ def main_loop():
             return
 
         for fd in rin:
+            if mpstate is None:
+                  return
             for master in mpstate.mav_master:
-                if fd == master.fd:
-                    process_master(master)
-                    continue
+                  if fd == master.fd:
+                        process_master(master)
+                        if mpstate is None:
+                              return
+                        continue
             for m in mpstate.mav_outputs:
                 if fd == m.fd:
                     process_mavlink(m)
+                    if mpstate is None:
+                          return
                     continue
 
             # this allow modules to register their own file descriptors
@@ -804,6 +822,8 @@ if __name__ == '__main__':
     parser.add_option("--dialect",  default="ardupilotmega", help="MAVLink dialect")
     parser.add_option("--rtscts",  action='store_true', help="enable hardware RTS/CTS flow control")
     parser.add_option("--mission", dest="mission", help="mission name", default=None)
+    parser.add_option("--daemon", action='store_true', help="run in daemon mode, do not start interactive shell")
+    parser.add_option("--profile", action='store_true', help="run the Yappi python profiler")
 
     (opts, args) = parser.parse_args()
 
@@ -829,21 +849,9 @@ if __name__ == '__main__':
 
     if not opts.master:
         serial_list = mavutil.auto_detect_serial(preferred_list=['*FTDI*',"*Arduino_Mega_2560*", "*3D_Robotics*", "*USB_to_UART*", '*PX4*', '*FMU*'])
-        if len(serial_list) == 1:
-            opts.master = [serial_list[0].device]
-        else:
-            print('''
-Please choose a MAVLink master with --master
-For example:
-    --master=com14
-    --master=/dev/ttyUSB0
-    --master=127.0.0.1:14550
-
-Auto-detected serial ports are:
-''')
-            for port in serial_list:
-                print("%s" % port)
-            sys.exit(1)
+        print('Auto-detected serial ports are:')
+        for port in serial_list:
+              print("%s" % port)
 
     # container for status information
     mpstate.status.target_system = opts.TARGET_SYSTEM
@@ -851,6 +859,28 @@ Auto-detected serial ports are:
 
     mpstate.mav_master = []
 
+    mpstate.rl = rline.rline("MAV> ", mpstate)
+
+    def quit_handler(signum = None, frame = None):
+        #print 'Signal handler called with signal', signum
+        if mpstate.status.exit:
+            print 'Clean shutdown impossible, forcing an exit'
+            sys.exit(0)
+        else:
+            mpstate.status.exit = True
+
+    # Listen for kill signals to cleanly shutdown modules
+    fatalsignals = [signal.SIGTERM]
+    try:
+          fatalsignals.append(signal.SIGHUP, signal.SIGQUIT)
+    except Exception:
+          pass
+    if opts.daemon: # SIGINT breaks readline parsing - if we are interactive, just let things die
+        fatalsignals.append(signal.SIGINT)
+
+    for sig in fatalsignals:
+        signal.signal(sig, quit_handler)
+
     load_module('link', quiet=True)
 
     # open master link
@@ -877,7 +907,6 @@ Auto-detected serial ports are:
 
     mpstate.input_queue = Queue.Queue()
     mpstate.input_count = 0
-    mpstate.rl = rline.rline("MAV> ", mpstate)
     if opts.setup:
         mpstate.rl.set_prompt("")
 
@@ -922,8 +951,12 @@ Auto-detected serial ports are:
             for c in cmds:
                 process_stdin(c)
 
+    if opts.profile:
+        import yappi    # We do the import here so that we won't barf if run normally and yappi not available
+        yappi.start()
+
     # run main loop as a thread
-    mpstate.status.thread = threading.Thread(target=main_loop)
+    mpstate.status.thread = threading.Thread(target=main_loop, name='main_loop')
     mpstate.status.thread.daemon = True
     mpstate.status.thread.start()
 
@@ -931,7 +964,10 @@ Auto-detected serial ports are:
     # up on exit
     while (mpstate.status.exit != True):
         try:
-            input_loop()
+            if opts.daemon:
+                time.sleep(0.1)
+            else:
+                input_loop()
         except KeyboardInterrupt:
             if mpstate.settings.requireexit:
                 print("Interrupt caught.  Use 'exit' to quit MAVProxy.")
@@ -945,12 +981,16 @@ Auto-detected serial ports are:
                             except Exception:
                                 pass
                         reload(m)
-                        m.init(mpstate)   
+                        m.init(mpstate)
 
             else:
                 mpstate.status.exit = True
                 sys.exit(1)
 
+    if opts.profile:
+        yappi.get_func_stats().print_all()
+        yappi.get_thread_stats().print_all()
+
     #this loop executes after leaving the above loop and is for cleanup on exit
     for (m,pm) in mpstate.modules:
         if hasattr(m, 'unload'):
