@@ -22,6 +22,7 @@ class ConsoleModule(mp_module.MPModule):
         self.start_time = 0.0
         self.total_time = 0.0
         self.speed = 0
+        self.max_link_num = 0
         mpstate.console = wxconsole.MessageConsole(title='Console')
 
         # setup some default status information
@@ -32,6 +33,7 @@ class ConsoleModule(mp_module.MPModule):
         mpstate.console.set_status('INS', 'INS', fg='grey', row=0)
         mpstate.console.set_status('MAG', 'MAG', fg='grey', row=0)
         mpstate.console.set_status('AS', 'AS', fg='grey', row=0)
+        mpstate.console.set_status('RNG', 'RNG', fg='grey', row=0)
         mpstate.console.set_status('AHRS', 'AHRS', fg='grey', row=0)
         mpstate.console.set_status('Heading', 'Hdg ---/---', row=2)
         mpstate.console.set_status('Alt', 'Alt ---', row=2)
@@ -41,6 +43,7 @@ class ConsoleModule(mp_module.MPModule):
         mpstate.console.set_status('Thr', 'Thr ---', row=2)
         mpstate.console.set_status('Roll', 'Roll ---', row=2)
         mpstate.console.set_status('Pitch', 'Pitch ---', row=2)
+        mpstate.console.set_status('Wind', 'Wind ---/---', row=2)
         mpstate.console.set_status('WP', 'WP --', row=3)
         mpstate.console.set_status('WPDist', 'Distance ---', row=3)
         mpstate.console.set_status('WPBearing', 'Bearing ---', row=3)
@@ -195,11 +198,12 @@ class ConsoleModule(mp_module.MPModule):
             self.console.set_status('Roll', 'Roll %u' % math.degrees(msg.roll))
             self.console.set_status('Pitch', 'Pitch %u' % math.degrees(msg.pitch))
         elif type in ['SYS_STATUS']:
-            sensors = { 'AS'  : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_DIFFERENTIAL_PRESSURE,
-                        'MAG' : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_3D_MAG,
-                        'INS' : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_3D_ACCEL | mavutil.mavlink.MAV_SYS_STATUS_SENSOR_3D_GYRO,
+            sensors = { 'AS'   : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_DIFFERENTIAL_PRESSURE,
+                        'MAG'  : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_3D_MAG,
+                        'INS'  : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_3D_ACCEL | mavutil.mavlink.MAV_SYS_STATUS_SENSOR_3D_GYRO,
                         'AHRS' : mavutil.mavlink.MAV_SYS_STATUS_AHRS,
-                        'TERR' : mavutil.mavlink.MAV_SYS_STATUS_TERRAIN}
+                        'TERR' : mavutil.mavlink.MAV_SYS_STATUS_TERRAIN,
+                        'RNG'  : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_LASER_POSITION}
             for s in sensors.keys():
                 bits = sensors[s]
                 present = ((msg.onboard_control_sensors_enabled & bits) == bits)
@@ -210,7 +214,14 @@ class ConsoleModule(mp_module.MPModule):
                     fg = 'red'
                 else:
                     fg = 'green'
+                # for terrain show yellow if still loading
+                if s == 'TERR' and fg == 'green' and master.field('TERRAIN_REPORT', 'pending', 0) != 0:
+                    fg = 'yellow'
                 self.console.set_status(s, s, fg=fg)
+
+        elif type == 'WIND':
+            self.console.set_status('Wind', 'Wind %u/%.2f' % (msg.direction, msg.speed))
+
         elif type == 'HWSTATUS':
             if msg.Vcc >= 4600 and msg.Vcc <= 5300:
                 fg = 'green'
@@ -243,6 +254,10 @@ class ConsoleModule(mp_module.MPModule):
             self.console.set_status('Radio', 'Radio %u/%u %u/%u' % (msg.rssi, msg.noise, msg.remrssi, msg.remnoise), fg=fg)
         elif type == 'HEARTBEAT':
             self.console.set_status('Mode', '%s' % master.flightmode, fg='blue')
+            if self.max_link_num != len(self.mpstate.mav_master):
+                for i in range(self.max_link_num):
+                    self.console.set_status('Link%u'%(i+1), '', row=1)
+                self.max_link_num = len(self.mpstate.mav_master)
             for m in self.mpstate.mav_master:
                 linkdelay = (self.mpstate.status.highest_msec - m.highest_msec)*1.0e-3
                 linkline = "Link %u " % (m.linknum+1)
@@ -250,7 +265,11 @@ class ConsoleModule(mp_module.MPModule):
                     linkline += "down"
                     fg = 'red'
                 else:
-                    linkline += "OK (%u pkts, %.2fs delay, %u lost)" % (m.mav_count, linkdelay, m.mav_loss)
+                    packets_rcvd_percentage = 100
+                    if (m.mav_loss != 0): #avoid divide-by-zero
+                        packets_rcvd_percentage = (1.0 - (float(m.mav_loss) / float(m.mav_count))) * 100.0
+
+                    linkline += "OK (%u pkts, %.2fs delay, %u lost) %u%%" % (m.mav_count, linkdelay, m.mav_loss, packets_rcvd_percentage)
                     if linkdelay > 1:
                         fg = 'orange'
                     else:
