@@ -12,13 +12,57 @@ import time, struct, math, sys, fnmatch, traceback
 from MAVProxy.modules.lib import mp_module
 from MAVProxy.modules.lib import mp_util
 
+if mp_util.has_wxpython:
+    from MAVProxy.modules.lib.mp_menu import *
+
+dataPackets = frozenset(['BAD_DATA','LOG_DATA'])
+delayedPackets = frozenset([ 'MISSION_CURRENT', 'SYS_STATUS', 'VFR_HUD',
+                  'GPS_RAW_INT', 'SCALED_PRESSURE', 'GLOBAL_POSITION_INT',
+                  'NAV_CONTROLLER_OUTPUT' ])
+activityPackets = frozenset([ 'HEARTBEAT', 'GPS_RAW_INT', 'GPS_RAW', 'GLOBAL_POSITION_INT', 'SYS_STATUS' ])
+
 class LinkModule(mp_module.MPModule):
+
     def __init__(self, mpstate):
         super(LinkModule, self).__init__(mpstate, "link", "link control", public=True)
         self.add_command('link', self.cmd_link, "link control",
-                         ["<list|add|remove>"])
+                         ["<list|ports>",
+                          'add (SERIALPORT)',
+                          'remove (LINKS)'])
         self.no_fwd_types = set()
         self.no_fwd_types.add("BAD_DATA")
+        self.add_completion_function('(SERIALPORT)', self.complete_serial_ports)
+        self.add_completion_function('(LINKS)', self.complete_links)
+
+        if mp_util.has_wxpython:
+            self.menu_added_console = False
+            self.menu_add = MPMenuSubMenu('Add', items=[])
+            self.menu_rm = MPMenuSubMenu('Remove', items=[])
+            self.menu = MPMenuSubMenu('Link',
+                                      items=[self.menu_add,
+                                             self.menu_rm,
+                                             MPMenuItem('Ports', 'Ports', '# link ports'),
+                                             MPMenuItem('List', 'List', '# link list'),
+                                             MPMenuItem('Status', 'Status', '# link')])
+            self.last_menu_update = 0
+
+    def idle_task(self):
+        '''called on idle'''
+        if not self.menu_added_console and self.module('console') is not None:
+            self.menu_added_console = True
+            # we don't dynamically update these yet due to a wx bug
+            self.menu_add.items = [ MPMenuItem(p, p, '# link add %s' % p) for p in self.complete_serial_ports('') ]
+            self.menu_rm.items = [ MPMenuItem(p, p, '# link remove %s' % p) for p in self.complete_links('') ]
+            self.module('console').add_menu(self.menu)
+
+    def complete_serial_ports(self, text):
+        '''return list of serial ports'''
+        ports = mavutil.auto_detect_serial(preferred_list=['*FTDI*',"*Arduino_Mega_2560*", "*3D_Robotics*", "*USB_to_UART*", '*PX4*', '*FMU*'])
+        return [ p.device for p in ports ]
+
+    def complete_links(self, text):
+        '''return list of links'''
+        return [ m.address for m in self.mpstate.mav_master ]
 
     def cmd_link(self, args):
         '''handle link commands'''
@@ -31,6 +75,8 @@ class LinkModule(mp_module.MPModule):
                 print("Usage: link add LINK")
                 return
             self.cmd_link_add(args[1:])
+        elif args[0] == "ports":
+            self.cmd_link_ports()
         elif args[0] == "remove":
             if len(args) != 2:
                 print("Usage: link remove LINK")
@@ -91,6 +137,12 @@ class LinkModule(mp_module.MPModule):
         print("Adding link %s" % device)
         self.link_add(device)
 
+    def cmd_link_ports(self):
+        '''show available ports'''
+        ports = mavutil.auto_detect_serial(preferred_list=['*FTDI*',"*Arduino_Mega_2560*", "*3D_Robotics*", "*USB_to_UART*", '*PX4*', '*FMU*'])
+        for p in ports:
+            print("%s : %s : %s" % (p.device, p.description, p.hwid))
+
     def cmd_link_remove(self, args):
         '''remove an link'''
         device = args[0]
@@ -189,14 +241,14 @@ class LinkModule(mp_module.MPModule):
         mtype = m.get_type()
 
         # and log them
-        if mtype not in ['BAD_DATA','LOG_DATA'] and self.mpstate.logqueue:
+        if mtype not in dataPackets and self.mpstate.logqueue:
             # put link number in bottom 2 bits, so we can analyse packet
             # delay in saved logs
             usec = self.get_usec()
             usec = (usec & ~3) | master.linknum
             self.mpstate.logqueue.put(str(struct.pack('>Q', usec) + m.get_msgbuf()))
 
-        if mtype in [ 'HEARTBEAT', 'GPS_RAW_INT', 'GPS_RAW', 'GLOBAL_POSITION_INT', 'SYS_STATUS' ]:
+        if mtype in activityPackets:
             if master.linkerror:
                 master.linkerror = False
                 self.say("link %u OK" % (master.linknum+1))
@@ -205,9 +257,7 @@ class LinkModule(mp_module.MPModule):
 
         if master.link_delayed:
             # don't process delayed packets that cause double reporting
-            if mtype in [ 'MISSION_CURRENT', 'SYS_STATUS', 'VFR_HUD',
-                          'GPS_RAW_INT', 'SCALED_PRESSURE', 'GLOBAL_POSITION_INT',
-                          'NAV_CONTROLLER_OUTPUT' ]:
+            if mtype in delayedPackets:
                 return
 
         if mtype == 'HEARTBEAT' and m.get_srcSystem() != 255:
@@ -241,7 +291,7 @@ class LinkModule(mp_module.MPModule):
                 self.mpstate.rl.set_prompt(self.status.flightmode + "> ")
                 self.say("Mode " + self.status.flightmode)
 
-            if m.type in [mavutil.mavlink.MAV_TYPE_FIXED_WING]:
+            if m.type == mavutil.mavlink.MAV_TYPE_FIXED_WING:
                 self.mpstate.vehicle_type = 'plane'
                 self.mpstate.vehicle_name = 'ArduPlane'
             elif m.type in [mavutil.mavlink.MAV_TYPE_GROUND_ROVER,
@@ -336,7 +386,7 @@ class LinkModule(mp_module.MPModule):
         self.status.msg_count[m.get_type()] += 1
 
         # don't pass along bad data
-        if not mtype in ['BAD_DATA']:
+        if mtype != 'BAD_DATA':
             # pass messages along to listeners, except for REQUEST_DATA_STREAM, which
             # would lead a conflict in stream rate setting between mavproxy and the other
             # GCS
