@@ -43,6 +43,8 @@ class ParamState:
     def fetch_check(self, master):
         '''check for missing parameters periodically'''
         if self.param_period.trigger():
+            if master is None:
+                return
             if len(self.mav_param_set) == 0:
                 master.param_fetch_all()
             elif self.mav_param_count != 0 and len(self.mav_param_set) != self.mav_param_count:
@@ -110,7 +112,7 @@ class ParamState:
             else:
                 print("Parameter '%s' not found in documentation" % h)
 
-    def handle_command(self, master, args):
+    def handle_command(self, master, mpstate, args):
         '''handle parameter commands'''
         param_wildcard = "*"
         usage="Usage: param <fetch|set|show|load|preload|forceload|diff|download|help>"
@@ -168,6 +170,11 @@ class ParamState:
                 print("Unable to find parameter '%s'" % param)
                 return
             self.mav_param.mavset(master, param.upper(), value, retries=3)
+
+            if (param.upper() == "WP_LOITER_RAD" or param.upper() == "LAND_BREAK_PATH"):
+                #need to redraw rally points
+                mpstate.module('rally').rallyloader.last_change = time.time()
+
         elif args[0] == "load":
             if len(args) < 2:
                 print("Usage: param load <filename> [wildcard]")
@@ -210,7 +217,7 @@ class ParamModule(mp_module.MPModule):
         super(ParamModule, self).__init__(mpstate, "param", "parameter handling", public = True)
         self.pstate = ParamState(self.mav_param, self.logdir, self.vehicle_name, 'mav.parm')
         self.add_command('param', self.cmd_param, "parameter handling",
-                         ["<fetch|download>",
+                         ["<download>",
                           "<set|show|fetch|help> (PARAMETER)",
                           "<load|save|diff> (FILENAME)"])
         if self.continue_mode and self.logdir != None:
@@ -229,7 +236,7 @@ class ParamModule(mp_module.MPModule):
 
     def cmd_param(self, args):
         '''control parameters'''
-        self.pstate.handle_command(self.master, args)
+        self.pstate.handle_command(self.master, self.mpstate, args)
 
 def init(mpstate):
     '''initialise module'''
